import os
from os import sep
from glob import glob
import pandas as pd
import numpy as np
import bisect
from datetime import datetime, timedelta
from microt_prompt_matrix import utils
from microt_prompt_matrix.utils.get_time_diff import *

target_file_pattern = 'phone_detected_activity_clean*.csv'
detected_activity_columns = ["IN_VEHICLE", "ON_BIKE", "ON_FOOT", "RUNNING", "STILL", "TILTING", "WALKING", "UNKNOWN"]


def validate_dates_before_after(intermediate_participant_path, date_in_study):
    validated_date_list = []

    # target date
    date_folder_path = intermediate_participant_path + sep + date_in_study
    target_date_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
    if len(target_date_log_paths) == 0:
        print("No daily file of phone detected activity on {}".format(date_in_study))
    else:
        # 1 day before target date
        date_format = "%Y-%m-%d"
        one_date_before_datetime = datetime.strptime(date_in_study, date_format).date() - timedelta(days=1)
        one_date_before = one_date_before_datetime.strftime(date_format)
        date_folder_path = intermediate_participant_path + sep + one_date_before
        one_day_before_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(one_day_before_log_paths) != 0:
            validated_date_list.append(one_date_before)

        # target date
        validated_date_list.append(date_in_study)

        # 1 day after target date
        date_format = "%Y-%m-%d"
        one_date_after_datetime = datetime.strptime(date_in_study, date_format).date() + timedelta(days=1)
        one_date_after = one_date_after_datetime.strftime(date_format)
        date_folder_path = intermediate_participant_path + sep + one_date_after
        one_day_after_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(one_day_after_log_paths) != 0:
            validated_date_list.append(one_date_after)

    return validated_date_list


def clean_dataframe(df):
    df.reset_index(inplace=True, drop=True)
    dropped_rows = []
    for idx in df.index:
        local_time_str = df["LOG_TIME"][idx]
        if (local_time_str == "-1") or len(local_time_str.split(' ')) > 3 or len(
                local_time_str.split('-')[0]) > 4 or len(local_time_str.split(' ')) < 2:
            dropped_rows.append(idx)
    df = df.drop(dropped_rows)

    df.reset_index(inplace=True, drop=True)
    return df


def combine_intermediate_file(intermediate_participant_path, date_in_study):
    df_logs_combined = pd.DataFrame()
    participant_id = utils.extract_participant_id.extract_participant_id(intermediate_participant_path)

    # generate date range where date folder exists (sharable code in utils)
    validated_date_list = validate_dates_before_after(intermediate_participant_path, date_in_study)
    if len(validated_date_list) == 0:
        print("Cannot find logs file around {}".format(date_in_study))
        return df_logs_combined

    for date in validated_date_list:
        date_folder_path = intermediate_participant_path + sep + date
        csv_path_list = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name

        csv_path = csv_path_list[0]
        try:
            df_day = pd.read_csv(csv_path)
        except pd.errors.EmptyDataError:
            raise Exception("pandas.errors.EmptyDataError (phone detected activity) : " + csv_path)

        if df_day.shape[0] > 0:
            df_day['Participant_ID'] = [participant_id] * df_day.shape[0]
            df_logs_combined = pd.concat([df_logs_combined, df_day])

    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
    df_logs_combined = df_logs_combined.dropna(subset=['LOG_TIME'])
    df_logs_combined = clean_dataframe(df_logs_combined)

    try:
        df_logs_combined["Local_Time"] = [x.split(" ")[0] + " " + x.split(" ")[1] for x in
                                          list(df_logs_combined["LOG_TIME"])]

        # converter2 = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
        df_logs_combined['Local_Timestamp'] = pd.Series(map(converter, df_logs_combined["Local_Time"]))
        # print(df_logs_combined['Local_Timestamp'])
        df_logs_combined['Date'] = df_logs_combined['Local_Timestamp'].dt.date
    except IndexError:
        raise Exception(
            "IndexError: list index out of range (phone detected activity) : " + intermediate_participant_path + sep + date_in_study + str(
                list(df_logs_combined["LOG_TIME"])))
    except Exception:
        raise Exception("Exception (phone detected activity) : " + intermediate_participant_path + sep + date_in_study)

    return df_logs_combined


def find_closest_time(prompt_time, subset_time_list):
    i = bisect.bisect_left(subset_time_list, prompt_time)
    closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
    return closet_time


def match_feature(prompt_local_datetime_series, df_logs_combined):
    print("     --- start matching")
    detected_activity_list = []
    matched_time_list = []

    for idx in prompt_local_datetime_series.index:
        prompt_time = prompt_local_datetime_series[idx]
        # prompt_date = prompt_time.date()

        if df_logs_combined.shape[0] == 0:
            detected_activity = "NF"
            closest_time = "NF"
        else:
            subset_time_list = list(df_logs_combined["Local_Timestamp"])

            closest_time = find_closest_time(prompt_time, subset_time_list)

            # check if matched time is 5 minutes away from prompt time
            # if get_min_diff(prompt_time, closest_time) < 5:
            df_closest = df_logs_combined[df_logs_combined['Local_Timestamp'] == closest_time][
                detected_activity_columns]
            #             detected_activity = list(df_closest.idxmax(axis=1))[0]
            detected_activity = \
            list(df_closest.eq(df_closest.max(1), axis=0).dot(df_closest.columns + ',').str.rstrip(','))[0]
            # else:
            #     detected_activity = "NF"

        matched_time_list.append(closest_time)
        detected_activity_list.append(detected_activity)

    return detected_activity_list, matched_time_list


def transform(battery_level_column):
    # return [1 if x >= 15 else 0 for x in battery_level_column]
    return battery_level_column


def create_column(prompt_local_datetime_series, intermediate_participant_path, date_in_study):
    print("\n> start generating the feature: phone detected activity ")

    # Read, parse and combine related intermediate file
    df_logs_combined = combine_intermediate_file(intermediate_participant_path, date_in_study)

    if df_logs_combined.shape[0] > 0:
        # Match the combined parsed intermediate file with prompt feature data frame
        detected_activity_column, match_time = match_feature(prompt_local_datetime_series, df_logs_combined)
    else:
        detected_activity_column = ["NF"] * len(prompt_local_datetime_series)
        match_time = ["NF"] * len(prompt_local_datetime_series)

    # transform feature
    phone_detected_activity_column_transformed = transform(detected_activity_column)
    print("     --- success")

    return phone_detected_activity_column_transformed, match_time



from os import listdir, sep
import re


def findExistedHourFolder(hours_list):
    existed_folder_list = [x for x in hours_list if bool(re.match(r'\d{2}-[A-Z]{3}(\+[0-9_]{5}|$)', x))
                           or bool(re.match(r'\d{2}-[A-Z]{3}-\d{4}(\+[0-9_]{5}|$)', x))]
    return existed_folder_list


def validate_hours(date_folder_path):
    hours_list = listdir(date_folder_path)
    if len(hours_list) == 0:
        return []

    hours_list_existed = findExistedHourFolder(hours_list)

    HAVE_ALL_HOURS = True
    if len(hours_list_existed) != 24:
        HAVE_ALL_HOURS = False

    return hours_list_existed, HAVE_ALL_HOURS


if __name__ == "__main__":
    area_folder_path = r"E:\data\wocket\Wockets-win32-x64\resources\app\src\srv\MICROT\aditya4_internal@timestudy_com\logs-watch"
    date = listdir(area_folder_path)[1]
    date_folder_path = area_folder_path + sep + date

    validated_hour_list, HAVE_ALL_HOURS = validate_hours(date_folder_path)
    print(date)
    print("Total number of valid hour folders  :  {}".format(len(validated_hour_list)))
    print("Does the date have 24 hour folders : {}".format(HAVE_ALL_HOURS))
    print(validated_hour_list)


"""
Functionality: Parse the content of .../logs-watch/PromptResponses.log.csv for individual participant into formatted
matrices day by day and save as csv file in the directory .../participant_id@timestudy_com/appended_response.

Usage: This script is intended mainly to be called by uema_all_features_dataframe.py.

Input: python preprocess_promptResponse_raw.py [PARTICIPANT_ID] [microT_root_path] [start_date]
e.g., python preprocess_promptResponse_raw.py aditya4_internal .../MICROT 2020-01-01

Output: csv file for each day of the participant, with each row indicating a prompted question and response
e.g., aditya4_internal_uEMA_****-**-**.csv in .../aditya4_internal@timestudy_com/appended_response
"""
import os
import sys

import pandas.errors

from microt_prompt_matrix.utils.convert_timestamp import *
from glob import glob

columns_list = ["Participant_ID", "Initial_Prompt_Date", "Prompt_Type", "Study_Mode", "Initial_Prompt_Local_Time", "Initial_Prompt_UnixTime",
                "Initial_Prompt_UTC_Offset ",
                "Answer_Status", 'Question_Set_Completion_Local_Time', 'Question_Set_Completion_UnixTime',
                'Reprompt1_Prompt_Date',
                'Reprompt1_Prompt_Local_Time', 'Reprompt1_Prompt_UnixTime',
                'Reprompt2_Prompt_Date', 'Reprompt2_Prompt_Local_Time',
                'Reprompt2_Prompt_UnixTime ', 'Reprompt3_Prompt_Date',
                'Reprompt3_Prompt_Local_Time', 'Reprompt3_Prompt_UnixTime',
                'Reprompt4_Prompt_Date', 'Reprompt4_Prompt_Local_Time',
                'Reprompt4_Prompt_UnixTime', 'Reprompt5_Prompt_Date',
                'Reprompt5_Prompt_Local_Time', 'Reprompt5_Prompt_UnixTime', "Question_1_Answer_Unixtime"]

prompt_type_excluded = ['None_Sleep', 'None_Empty']


def extract_reprompt(df_pr_phone):
    prompt_arrays = []
    for idx in df_pr_phone.index:
        row = df_pr_phone.loc[idx]
        # How it works: for status repeated for both init prompt and reprompts, add the label to all prompts;
        # for completed prompts, label "neverStarted" to the prompts before the last one.
        if row["Answer_Status"] in ["NeverStarted", "Started", "NeverPrompted"]:
            # non-completed

            # init prompt
            new_reprompt_row = []
            new_reprompt_row += list(row[["Participant_ID", "Initial_Prompt_Date", "Line", 'Prompt_Type', 'Study_Mode']])
            new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
            new_reprompt_row.append(row["Answer_Status"])
            new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
            new_reprompt_row.append("null")
            new_reprompt_row.append(row["Initial_Prompt_UTC_Offset "])
            new_reprompt_row.append("0")
            prompt_arrays.append(new_reprompt_row)

            # reprompt
            for i in range(1, 6):
                reprompt_num = "Reprompt" + str(i)
                reprompt_unix = reprompt_num + "_Prompt_UnixTime"
                reprompt_local = reprompt_num + "_Prompt_Local_Time"
                if row[reprompt_unix] == -1:
                    break
                else:
                    new_reprompt_row = []
                    new_reprompt_row += list(row[["Participant_ID", "Initial_Prompt_Date", "Line", 'Prompt_Type', 'Study_Mode']])
                    new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
                    new_reprompt_row.append(row["Answer_Status"])
                    new_reprompt_row.append(row[reprompt_local])
                    new_reprompt_row.append(row["Question_1_Answer_Unixtime"])
                    new_reprompt_row.append(row["Initial_Prompt_UTC_Offset "])
                    new_reprompt_row.append(str(i))
                    prompt_arrays.append(new_reprompt_row)

        else:
            # completed
            for i in range(1, 6):
                reprompt_num = "Reprompt" + str(i)
                reprompt_unix = reprompt_num + "_Prompt_UnixTime"
                if row[reprompt_unix] == -1:
                    last_reprompt_num = i - 1
                    break

            if last_reprompt_num == 0:
                # init prompt completed
                new_reprompt_row = []
                new_reprompt_row += list(row[["Participant_ID", "Initial_Prompt_Date", "Line", 'Prompt_Type', 'Study_Mode']])
                new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
                new_reprompt_row.append(row["Answer_Status"])
                new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
                new_reprompt_row.append(row["Question_1_Answer_Unixtime"])
                new_reprompt_row.append(row["Initial_Prompt_UTC_Offset "])
                new_reprompt_row.append("0")
                prompt_arrays.append(new_reprompt_row)
                continue
            else:
                for i in range(0, last_reprompt_num + 1):

                    if i == 0:
                        status = "NeverStarted"
                        new_reprompt_row = []
                        new_reprompt_row += list(row[["Participant_ID", "Initial_Prompt_Date", "Line", 'Prompt_Type', 'Study_Mode']])
                        new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
                        new_reprompt_row.append(status)
                        new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
                        new_reprompt_row.append("null")
                        new_reprompt_row.append(row["Initial_Prompt_UTC_Offset "])
                        new_reprompt_row.append("0")
                        prompt_arrays.append(new_reprompt_row)

                    else:
                        if i == last_reprompt_num:
                            status = row["Answer_Status"]
                        else:
                            status = "NeverStarted"

                        reprompt_num = "Reprompt" + str(i)
                        reprompt_unix = reprompt_num + "_Prompt_UnixTime"
                        reprompt_local = reprompt_num + "_Prompt_Local_Time"

                        new_reprompt_row = []
                        new_reprompt_row += list(row[["Participant_ID", "Initial_Prompt_Date", "Line", 'Prompt_Type', 'Study_Mode']])
                        new_reprompt_row.append(row["Initial_Prompt_Local_Time"])
                        new_reprompt_row.append(status)
                        new_reprompt_row.append(row[reprompt_local])
                        new_reprompt_row.append(row["Question_1_Answer_Unixtime"])
                        new_reprompt_row.append(row["Initial_Prompt_UTC_Offset "])
                        new_reprompt_row.append(str(i))
                        prompt_arrays.append(new_reprompt_row)

    df_prompt = pd.DataFrame(prompt_arrays)
    if df_prompt.shape[0] > 0:
        df_prompt.columns = ["Participant_ID", "Initial_Prompt_Date", "Line", 'Prompt_Type', 'Study_Mode', 'Initial_Prompt_Local_Time', 'Answer_Status', "Actual_Prompt_Local_Time", 'First_Question_Completion_Unixtime', 'UTC_Offset', "Reprompt_Num"]

    return df_prompt


def clean_response_files(intermediate_participant_path, date_in_study, participant_id):
    # Check if date folder exists
    date_folder_path = os.path.join(intermediate_participant_path, date_in_study)
    # date_exist = validate_date(date_folder_path)
    # if not date_exist:
    #     print("Cannot find date folder for {}".format(date_in_study))
    #     raise Exception("Cannot find date folder for {}".format(date_in_study))

    # Check if phone_promptresponse file exists for this date
    csv_path = list(glob(os.path.join(date_folder_path, 'phone_promptresponse_clean*.csv')))
    if len(csv_path) == 0:
        print(date_folder_path)
        print("No intermediate promptresponse csv file on {}".format(date_in_study))
        quit()

    # read and filter prompt type
    try:
        df_raw = pd.read_csv(csv_path[0])
    except pandas.errors.EmptyDataError:
        raise Exception("pandas.errors.EmptyDataError (prompt response) : " + csv_path[0])


    if df_raw.shape[0] == 0:
        print("Empty prompt response file : " + csv_path[0])
        quit()

    if "Question_1_Answer_Unixtime" not in df_raw:
        print("All Prompts are Never Prompted {}".format(date_folder_path))
        quit()

    df_raw = df_raw[columns_list]

    if "Reprompt2_Prompt_UnixTime " in df_raw.columns:
        df_raw = df_raw.rename(columns={"Reprompt2_Prompt_UnixTime ": "Reprompt2_Prompt_UnixTime"})

    # add line number as prompt key. First prompt = 1.
    df_raw["Line"] = np.arange(1, len(df_raw)+1)

    df_raw = df_raw[~df_raw.Prompt_Type.isin(prompt_type_excluded)]
    df_raw.reset_index(drop=True, inplace=True)

    # extract reprompts and treated them as separate prompts
    df_reprompt = extract_reprompt(df_raw)

    if df_reprompt.shape[0] == 0:
        print("Empty prompt response file after filtering : " + csv_path[0])
        quit()

    # add participant id column
    # df_reprompt.insert(loc=0, column='Participant_ID', value=participant_id)

    # time zone processing
    df_reprompt.reset_index(drop=True, inplace=True)
    Prompt_Local_Time_list = []
    for idx in df_reprompt.index:
        prompt_local_time_timezone = df_reprompt['Actual_Prompt_Local_Time'][idx]
        prompt_local_time_timezone_split = prompt_local_time_timezone.split(" ")
        prompt_local_time_timezone_split_len = len(prompt_local_time_timezone_split)
        if prompt_local_time_timezone_split_len == 3:
            Prompt_Local_Time_list.append(prompt_local_time_timezone_split[0] + " " + prompt_local_time_timezone_split[1])
        else:
            print("Prompt response time zone error : " + intermediate_participant_path + "//" + date_in_study)
            Prompt_Local_Time_list.append(prompt_local_time_timezone)

    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
    prompt_dateobject_column = list(map(converter, Prompt_Local_Time_list))
    df_reprompt['Actual_Prompt_Local_DateTime'] = prompt_dateobject_column

    return df_reprompt


if __name__ == "__main__":
    p_id = sys.argv[1]
    intermediate_root_path = sys.argv[2]
    feature_save_path = sys.argv[3]
    start_date = sys.argv[4]
    end_date = sys.argv[5]
    clean_response_files(p_id, intermediate_root_path, feature_save_path, start_date, end_date)


"""
Output: a list of binary values of prompt compliance
e.g., [0,0,0,1,1,1]
"""

from collections import Counter

completion_list = ["Completed"]


def print_stats_single_column(feature_name, data):
    print('--------- Stats of Feature: ' + feature_name + ' ---------')
    counter = Counter(data)
    print(sorted(counter.items()))


def get_compliance_value(completion_str):

    # value = 0
    # if completion_str in completion_list:
    #     value = 1

    # return value
    return completion_str

def create_column(completion_status_column, print_stats=True):
    print("\n> start generating the feature: compliance ")
    # prompt_compliance_column = list(map(lambda x: get_compliance_value(x), completion_status_column))

    # if print_stats == True:
    #     print_stats_single_column("compliance", prompt_compliance_column)

    return completion_status_column


"""
Output: a list of categorical values indicating categories of study mode
e.g., [2,1,1,1,1]
"""

from collections import Counter


def print_stats_single_column(feature_name, data):
    print('--------- Stats of Feature: ' + feature_name + ' ---------')
    counter = Counter(data)
    print(sorted(counter.items()))


def get_mode_value(mode_str):
    value = 0
    if mode_str == "TIME":
        value = 1
    elif mode_str == "BURST":
        value = 2

    return value


def create_column(prompt_df, print_stats=True):
    study_mode_column = prompt_df['Study_Mode']
    study_mode_column = list(map(lambda x: get_mode_value(x), study_mode_column))

    if print_stats == True:
        print_stats_single_column("study mode", study_mode_column)

    return study_mode_column



import pandas as pd
import json
from microt_prompt_matrix.utils.uncompress_file import *


def check_inside_bbox(location, bbox):
    isInside = False

    lat = location[0]
    lon = location[1]

    bbox[0] = bbox[0]
    bbox[1] = bbox[1]
    bbox[2] = bbox[2]
    bbox[3] = bbox[3]

    if (lat >= bbox[0]) & (lat <= bbox[1]) & (lon >= bbox[2]) & (lon <= bbox[3]):
        isInside = True

    return isInside


def read_json(json_path):
    with open(json_path) as f:
        data = json.load(f)
    return data


def transform(location_column):
    return location_column


def find_cluster_for_loc(loc, lsf_json):
    matched_clusters = []
    for clusterID in lsf_json['bbox']:
        isInside = check_inside_bbox(loc, lsf_json['bbox'][clusterID])
        if isInside:
            matched_clusters.append(clusterID)
    return matched_clusters


def find_label_for_cluster(matched_clusters, lsf_json):
    labels = lsf_json["label"]
    matched_loc_label = []
    for clusterID in matched_clusters:
        if clusterID not in labels:
            continue
        if len(labels[clusterID]) == 0:
            continue
        b_s = pd.Series(labels[clusterID])
        if len(b_s) > 0:
            b = b_s.value_counts().index[0]  # take the mode label
            matched_loc_label.append(b)
    return matched_loc_label


def create_column(lsf_json_folder_path, participant_id, location_list):
    print("\n> start generating the feature: Location Label")

    # Loading lsf json
    lsf_json_path = os.path.join(lsf_json_folder_path, participant_id, "lsf.json")
    if not os.path.exists(lsf_json_path):
        print("     --- failure : no json file found for {}".format(participant_id))
        return ["NF"] * len(location_list)
    else:
        lsf_json = read_json(lsf_json_path)

    if ("bbox" not in lsf_json) or ("label" not in lsf_json):
        print("     --- failure : incomplete json file found for {}".format(participant_id))
        return ["NF"] * len(location_list)

    # map loc to cluster
    location_label_column = []
    for loc in location_list:
        loc_label = "NF"
        if loc not in ["NF", "OB"]:
            matched_clusters = find_cluster_for_loc(loc, lsf_json)
            loc_label = find_label_for_cluster(matched_clusters, lsf_json)
            loc_label = list(set(loc_label))
        location_label_column.append(loc_label)

    # transform feature
    location_column_transformed = transform(location_label_column)
    print("     --- success")

    return location_column_transformed


if __name__ == "__main__":
    location_list = [[47.60958, -122.33052]]
    lsf_json_folder_path = r"C:\Users\Jixin\Documents\testing\compliance"
    participant_id = "cat"
    print(create_column(lsf_json_folder_path, participant_id, location_list))


"""
Output: 1) a list of targeted features for the participant 2) print stats of the targeted features for the participant
e.g., [6,6,6,0,0,0,1,1,1]
"""

from collections import Counter


def print_stats_single_column(feature_name, data):
    print('--------- Stats of Feature: ' + feature_name + ' ---------')
    counter = Counter(data)
    print(sorted(counter.items()))


def create_column(prompt_local_datetime_series, print_stats=True):
    print("\n> start generating the feature: day of the week ")

    # map from date object to weekday
    prompt_weekday_column = list(map(lambda x: x.weekday(), prompt_local_datetime_series))

    if print_stats == True:
        print_stats_single_column("weekday", prompt_weekday_column)

    return prompt_weekday_column


"""
Output: 1) a list of targeted features for the participant 2) print stats of the targeted features for the participant
"""

from datetime import datetime
import numpy as np


def calculate_daysInStudy(prompte_date, start_date):
    delta = prompte_date - start_date
    return delta

# def get_start_date_dict():
#     # change to your path of combined_report.csv
#     combined_report_path = r"E:\compliance_analysis\report\combined_report.csv"
#     df_report = pd.read_csv(combined_report_path)
#     participants_list = df_report['participant_ID'].unique()
#     date_format = '%m/%d/%Y'
#     start_date_dict = {}
#     for p_id in participants_list:
#         start_date_str = list(df_report[df_report['participant_ID'] == p_id]['start_date'])[0]
#         start_date = datetime.strptime(start_date_str, date_format).date()
#         start_date_dict[p_id] = start_date
#     return start_date_dict


def create_column(df_prompt_combined, info_dict, print_stats=True):
    print("\n> start generating the feature: days in study ")
    participant_id = df_prompt_combined['Participant_ID'][0]
    series_len = len(df_prompt_combined.Actual_Prompt_Local_DateTime)

    if participant_id not in info_dict:
        # raise Exception("Don't find this participant in info json. Check if this participant has data, and delete Misc and rerun the code. ")
        days_in_study = [np.nan] * series_len
    else:
        if "start_date" in info_dict[participant_id]:
            start_date = info_dict[participant_id]["start_date"]
            start_date_dateobject = datetime.strptime(start_date, "%Y-%m-%d").date()
            prompt_dateobject_date_series = df_prompt_combined.Actual_Prompt_Local_DateTime.dt.date
            days_in_study = prompt_dateobject_date_series.apply(
                lambda x: calculate_daysInStudy(x, start_date_dateobject))
        else:
            days_in_study = [np.nan] * series_len
            # raise Exception(
            # "Don't find this participant in info json. Check if this participant has daily report, and delete Misc and rerun the code. ")

    return days_in_study

# def create_column(df_prompt_combined, info_dict, print_stats=True):
#     print("\n> start generating the feature: days in study ")
#     participant_id = df_prompt_combined['Participant_ID'][0]
#
#     if participant_id not in info_dict:
#         raise Exception("Don't find this participant in info json. Check if this participant has data, and delete Misc and rerun the code. ")
#     else:
#         if "start_date" in info_dict[participant_id]:
#             start_date = info_dict[participant_id]["start_date"]
#             start_date_dateobject = datetime.strptime(start_date, "%Y-%m-%d").date()
#         else:
#             raise Exception(
#                 "Don't find this participant in info json. Check if this participant has daily report, and delete Misc and rerun the code. ")
#
#     try:
#         prompt_dateobject_date_series = df_prompt_combined.Actual_Prompt_Local_DateTime.dt.date
#     except AttributeError:
#         raise AttributeError("AttributeError : {} - {}".format(participant_id, df_prompt_combined.Actual_Prompt_Local_DateTime))
#
#     days_in_study = prompt_dateobject_date_series.apply(lambda x: calculate_daysInStudy(x, start_date_dateobject))
#
#     return days_in_study


import pandas as pd
import datetime

import os
from os import listdir
from glob import glob

from sklearn.cluster import DBSCAN, OPTICS

import json
from tqdm import tqdm
import numpy as np

from collections import Counter


# get_home_boarder_coordinates() is the main function:
# Get the max Latitude, min Latitude, max Longitude, min Longitude coordinates of home for each participant.
# e.g., {'XXX': [40.82849023, 40.82956608, -73.62032694, -73.61905812]}

def get_home_boarder_coordinates(participant_ids, microt_root_path):
    participants_home_boarders = {}
    for p_id in tqdm(participant_ids):
        print(p_id)
        df_p = pd.DataFrame()
        participant_path = os.path.join(*[microt_root_path, "intermediate_file",  p_id])
        date_list = listdir(participant_path)

        for date in date_list:
            date_path = os.path.join(participant_path, date)
            csv_path = glob(os.path.join(date_path, 'phone_GPS_clean*.csv'))
            if len(csv_path) > 0:
                df = pd.read_csv(csv_path[0])
                df_p = pd.concat([df_p, df])
        boarder_coords = find_home_individual_box(df_p)
        participants_home_boarders[p_id] = boarder_coords
    return participants_home_boarders


def find_home_individual_box(df_p):
    # Step1: filter out night time (0-5am) location
    df_night_gps_r = subset_night_location(df_p)

    # Step2: find the largest cluster during night as the home cluster
    kms_per_radian = 6371.0088
    epsilon = .02 / kms_per_radian
    home_list = find_home_cluster(df_night_gps_r, epsilon)

    # Step3: find the four extreme coordinates of the bounding box of the home cluster
    df_night_gps = df_p[df_p["night"] == 1][['LAT', 'LONG']]
    boarder_coords = find_boarder_coordinates(df_night_gps, home_list)

    return boarder_coords


def subset_night_location(df_p):
    datetime_list = [datetime.datetime.strptime(x.strip(x.split(' ')[2]).strip(" "), '%Y-%m-%d %H:%M:%S') for x in
                     df_p['LOCATION_TIME']]

    hour_list = []
    for i in range(len(datetime_list)):
        hour_list.append(datetime_list[i].hour)

    night_list = [1 if x < 6 else 0 for x in hour_list]

    df_p["night"] = night_list
    df_p['lat_r'] = np.radians(df_p['LAT'])
    df_p['long_r'] = np.radians(df_p['LONG'])
    df_night_gps_r = df_p[df_p["night"] == 1][['lat_r', 'long_r']]
    df_night_gps_r.reset_index(inplace=True, drop=True)

    return df_night_gps_r


def find_home_cluster(df_night_gps_r, epsilon):
    coords = df_night_gps_r.values

    clusterer_dbscan = OPTICS(eps=epsilon, metric='haversine', cluster_method='dbscan', algorithm='ball_tree')
    clusterer_dbscan.fit(coords)
    cluster_result_list = clusterer_dbscan.labels_

    cluster_value = Counter(cluster_result_list).most_common(1)[0][0]
    home_list = [1 if x == cluster_value else 0 for x in cluster_result_list]

    return home_list


def find_boarder_coordinates(df_night_gps, home_list):
    df_night_gps['Home'] = home_list

    df_home = df_night_gps[df_night_gps['Home'] == 1]

    adjustment = 0.0001 * 3
    long_max = max(df_home['LONG']) + adjustment
    long_min = min(df_home['LONG']) - adjustment
    lat_max = max(df_home['LAT']) + adjustment
    lat_min = min(df_home['LAT']) - adjustment
    return [lat_min, lat_max, long_min, long_max]


if __name__ == "__main__":
    participant_ids = ["bondlesspessimistlivestock", "capitolreferableswinging", "conductorpushpindepth",
                       "dimnesscranialunheard", "eskimovocalizeveggie", "fontsixtyfoldvocally",
                       "gullyskinheadpleat", "hacksawscoldingdares", "reggaeascendactivity",
                       "shrankdicesprain", "shushsaddenenactment", "spideraffixreptilian"]
    microt_root_path = r"E:\intermediate_file"
    result = get_home_boarder_coordinates(participant_ids, microt_root_path)
    with open(r'E:\preliminary_data\OSM\home_boarder_coords.json', 'w') as fp:
        json.dump(result, fp, sort_keys=True, indent=4)


import setuptools

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setuptools.setup(
    name='microt_prompt',
    version="0.1.6",
    description='A package that transform intermediate log files into features of interest for analysis',
    url='https://bitbucket.org/mhealthresearchgroup/microt_compliance/src/master/',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Jixin Li, Aditya Ponnada',
    author_email='li.jix@northeastern.edu',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.7'
)


import os
from os import sep
from glob import glob
import pandas as pd
import numpy as np
import bisect
# from tqdm import tqdm
from datetime import datetime, timedelta
from microt_prompt_matrix import utils
from microt_prompt_matrix.utils.get_time_diff import *
from microt_prompt_matrix.utils.uncompress_file import *
from microt_prompt_matrix.utils.get_haversine_distance import *

target_file_pattern = 'phone_GPS_clean*.csv.zip'

def validate_dates_before_after(intermediate_participant_path, date_in_study):
    validated_date_list = []

    # target date
    date_folder_path = intermediate_participant_path + sep + date_in_study
    target_date_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
    if len(target_date_log_paths) == 0:
        print("No battery daily file on {}".format(date_in_study))
    else:

        # 1 day before target date
        date_format = "%Y-%m-%d"
        one_date_before_datetime = datetime.strptime(date_in_study, date_format).date() - timedelta(days=1)
        one_date_before = one_date_before_datetime.strftime(date_format)
        date_folder_path = intermediate_participant_path + sep + one_date_before
        one_day_before_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(one_day_before_log_paths) != 0:
            validated_date_list.append(one_date_before)

        # target date
        validated_date_list.append(date_in_study)

        # 1 day after target date
        date_format = "%Y-%m-%d"
        one_date_after_datetime = datetime.strptime(date_in_study, date_format).date() + timedelta(days=1)
        one_date_after = one_date_after_datetime.strftime(date_format)
        date_folder_path = intermediate_participant_path + sep + one_date_after
        one_day_after_log_paths = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(one_day_after_log_paths) != 0:
            validated_date_list.append(one_date_after)

    return validated_date_list

# def combine_intermediate_file(intermediate_participant_path, date_in_study, decryption_password):
#     df_logs_combined = pd.DataFrame()
#     participant_id = utils.extract_participant_id.extract_participant_id(intermediate_participant_path)
#
#     # generate date range where date folder exists (sharable code in utils)
#     validated_date_list = validate_dates_before_after(intermediate_participant_path, date_in_study)
#     if len(validated_date_list) == 0:
#         print("Cannot find logs file around {}".format(date_in_study))
#         return df_logs_combined
#
#     for date in validated_date_list:
#         date_folder_path = intermediate_participant_path + sep + date
#         csv_path_list = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
#
#         csv_path = csv_path_list[0]
#
#         # decryption
#         unzip_file_path = uncompress(csv_path, decryption_password)
#         print(unzip_file_path)
#         if not os.path.exists(unzip_file_path):
#             print("decrypt failed for : {}".format(unzip_file_path))
#             continue
#         try:
#             df_day = pd.read_csv(unzip_file_path)
#         except pd.errors.EmptyDataError:
#             print("pandas.errors.EmptyDataError (location) : " + unzip_file_path)
#             quit()
#
#         if df_day.shape[0] > 0:
#             df_day['Participant_ID'] = [participant_id] * df_day.shape[0]
#             df_logs_combined = pd.concat([df_logs_combined, df_day])
#         delete_unzipped_file(unzip_file_path)
#
#     converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
#     df_logs_combined = df_logs_combined.dropna(subset=['LOCATION_TIME'])
#     df_logs_combined.reset_index(inplace=True, drop=True)
#     df_logs_combined["LOCATION_TIME"] = [x.split(" ")[0] + " " + x.split(" ")[1] for x in
#                                          list(df_logs_combined["LOCATION_TIME"])]
#
#     df_logs_combined['LOCATION_TIME_DATETIME'] = pd.Series(map(converter, df_logs_combined["LOCATION_TIME"]))
#     df_logs_combined['Date'] = df_logs_combined['LOCATION_TIME_DATETIME'].dt.date
#
#     return df_logs_combined

def combine_intermediate_file(intermediate_participant_path, date_in_study, decryption_password):
    df_logs_combined = pd.DataFrame()
    participant_id = utils.extract_participant_id.extract_participant_id(intermediate_participant_path)

    # generate date range where date folder exists (sharable code in utils)
    # validated_date_list = validate_dates_before_after(intermediate_participant_path, date_in_study)
    # if len(validated_date_list) == 0:
    #     print("Cannot find logs file around {}".format(date_in_study))
    #     return df_logs_combined

    for date in [date_in_study]:
        date_folder_path = intermediate_participant_path + sep + date
        csv_path_list = sorted(glob(os.path.join(date_folder_path, target_file_pattern)))  # file name
        if len(csv_path_list) == 0:
            print("Cannot find logs file around {}".format(date_in_study))
            return df_logs_combined

        csv_path = csv_path_list[0]

        # decryption
        unzip_file_path = uncompress(csv_path, decryption_password)
        print(unzip_file_path)
        if not os.path.exists(unzip_file_path):
            print("decrypt failed for : {}".format(unzip_file_path))
            continue
        try:
            df_day = pd.read_csv(unzip_file_path)
        except pd.errors.EmptyDataError:
            print("pandas.errors.EmptyDataError (location) : " + unzip_file_path)
            return df_logs_combined

        if df_day.shape[0] > 0:
            df_day['Participant_ID'] = [participant_id] * df_day.shape[0]
            df_logs_combined = pd.concat([df_logs_combined, df_day])
        delete_unzipped_file(unzip_file_path)

    converter = lambda x: datetime.strptime(x, "%Y-%m-%d %H:%M:%S")
    df_logs_combined = df_logs_combined.dropna(subset=['LOCATION_TIME'])
    df_logs_combined.reset_index(inplace=True, drop=True)
    df_logs_combined["LOCATION_TIME"] = [x.split(" ")[0] + " " + x.split(" ")[1] for x in
                                         list(df_logs_combined["LOCATION_TIME"])]

    df_logs_combined['LOCATION_TIME_DATETIME'] = pd.Series(map(converter, df_logs_combined["LOCATION_TIME"]))
    df_logs_combined['Date'] = df_logs_combined['LOCATION_TIME_DATETIME'].dt.date

    return df_logs_combined

def find_closest_time(prompt_time, subset_time_list):
    i = bisect.bisect_left(subset_time_list, prompt_time)
    closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
    return closet_time


def get_distance_from_home(location, info_dict):
    distance_from_home = "bboxNF"  # this means the lsf.json has no bbox for home cluster (i.e., lsf.json recovery error)
    if type(location) != str:
        if "home_loc" in info_dict:
            home_coords = info_dict["home_loc"]
            for home_coord in home_coords:
                d_tmp = haversine(location, home_coord)
                if distance_from_home == "bboxNF":
                    distance_from_home = d_tmp
                if d_tmp < distance_from_home:
                    distance_from_home = d_tmp
        else:
            distance_from_home = "jsonNF"

    return distance_from_home


def match_feature(prompt_local_datetime_series, df_logs_combined, info_dict, participant_id):
    print("     --- start matching")
    matched_location_list = []
    matched_time_list = []
    distance_from_home_list = []
    p_id_exist_info_dict = True if participant_id in info_dict else False

    for idx in prompt_local_datetime_series.index:
        prompt_time = prompt_local_datetime_series[idx]
        # prompt_date = prompt_time.date()

        if df_logs_combined.shape[0] == 0:
            location = "NF"
            closest_time = "NF"
            distance_from_home = "NF"  # this means the location file is missing
        else:
            subset_time_list = list(df_logs_combined["LOCATION_TIME_DATETIME"])

            closest_time = find_closest_time(prompt_time, subset_time_list)

            # check if matched time is 5 minutes away from prompt time
            if get_min_diff(prompt_time, closest_time) < 5:
                lat = list(df_logs_combined[df_logs_combined['LOCATION_TIME_DATETIME'] == closest_time][
                               "LAT"])[0]
                long = list(df_logs_combined[df_logs_combined['LOCATION_TIME_DATETIME'] == closest_time][
                                "LONG"])[0]
                location = [lat, long]
            else:
                location = "OB"

        matched_time_list.append(closest_time)
        matched_location_list.append(location)
        if p_id_exist_info_dict:
            distance_from_home = get_distance_from_home(location, info_dict[participant_id])
        else:
            distance_from_home = "pidNF"  # this means this participant has no recovered lsf.json
        distance_from_home_list.append(distance_from_home)

    return matched_location_list, matched_time_list, distance_from_home_list


# def reverse_geo_nominatim(lat, long):
#     URL = "https://nominatim.openstreetmap.org/reverse?"
#     PARAMS = {'format': 'geojson', 'lat': lat, 'lon': long}
#     r = requests.get(url=URL, params=PARAMS)
#     result = r.json()
#
#     category = result['features'][0]['properties']['category']
#     type_loc = result['features'][0]['properties']['type']
#
#     location = [category, type_loc]
#     return location


def transform(location_column):
    return location_column


def create_column(participant_id, prompt_local_datetime_series, info_dict, intermediate_participant_path, date_in_study,
                  decryption_password):
    print("\n> start generating the feature: Location ")

    # Read, parse and combine related intermediate file
    df_logs_combined = combine_intermediate_file(intermediate_participant_path, date_in_study, decryption_password)

    if df_logs_combined.shape[0] > 0:
        # Match the combined parsed intermediate file with prompt feature data frame
        location_column, match_time, DFH_column = match_feature(prompt_local_datetime_series, df_logs_combined,
                                                                info_dict, participant_id)
    else:
        location_column = ["NF"] * len(prompt_local_datetime_series)
        match_time = ["NF"] * len(prompt_local_datetime_series)
        DFH_column = ["NF"] * len(prompt_local_datetime_series)

    # transform feature
    location_column_transformed = transform(location_column)
    print("     --- success")

    return location_column_transformed, match_time, DFH_column


if __name__ == "__main__":
    gps_list = [[42.340702, -71.117756]]
    print(transform(gps_list))